import cv2
import numpy as np
import matplotlib.pyplot as plt
from sklearn.cluster import KMeans
from os import listdir
import imageio

from PIL import Image, ImageDraw

def list_files(directory, extension):
    return (f for f in listdir(directory) if f.endswith('.jpg') or f.endswith('.png'))

def find_histogram(clt):
    """
    create a histogram with k clusters
    :param: clt
    :return:hist
    """
    numLabels = np.arange(0, len(np.unique(clt.labels_)) + 1)
    (hist, _) = np.histogram(clt.labels_, bins=numLabels)

    hist = hist.astype("float")
    hist /= hist.sum()

    return hist
def plot_colors2(hist, centroids):
    bar = np.zeros((50, 300, 3), dtype="uint8")
    startX = 0

    for (percent, color) in zip(hist, centroids):
        # plot the relative percentage of each cluster
        endX = startX + (percent * 300)
        cv2.rectangle(bar, (int(startX), 0), (int(endX), 50),
                      color.astype("uint8").tolist(), -1)
        startX = endX

    # return the bar chart
    return bar


def generateColorImage(img_input):
    img = cv2.imread("images/"+img_input)

    img = cv2.cvtColor(img, cv2.COLOR_BGR2RGB)
    img_old = img

    height=img.shape[0]
    width=img.shape[1]

    img = img.reshape((img.shape[0] * img.shape[1],3)) #represent as row*column,channel number
    clt = KMeans(n_clusters=3) #cluster number
    clt.fit(img)

    hist = find_histogram(clt)

    #print(hist, clt.cluster_centers_)

    # size of image
    canvas = (width, height)

    # scale ration
    scale = 1
    thumb = canvas[0]/scale, canvas[1]/scale

    # rectangles (width, height, left position, top position)
    #print(width*hist[0])
    frames = [(0, 0, 115, height), (width*hist[0], height, width*hist[0]+width,2), (100, 205, 120,200)]

    # init canvas
    im = Image.new('RGB', canvas, (255, 255, 255))
    draw = ImageDraw.Draw(im)

    draw.rectangle([0, 0, width*hist[0], height], fill=(int(clt.cluster_centers_[0][0]), int(clt.cluster_centers_[0][1]), int(clt.cluster_centers_[0][2])))
    draw.rectangle([width*hist[0], 0, width*hist[0]+width*hist[1], height], fill=(int(clt.cluster_centers_[1][0]),int(clt.cluster_centers_[1][1]),int(clt.cluster_centers_[1][2])))
    draw.rectangle([width*hist[0]+width*hist[1], 0, width*hist[0]+width*hist[1]+width*hist[2], height], fill=(int(clt.cluster_centers_[2][0]),int(clt.cluster_centers_[2][1]),int(clt.cluster_centers_[2][2])))

    # make thumbnail
    im.thumbnail(thumb)

    # save image
    im.save("output/"+img_input)

    # images = []
    # images.append(imageio.imread("images/"+img_input))
    # images.append(imageio.imread("img/"+img_input))
    # imageio.mimsave('gifs/'+img_input+".gif", images)



directory = 'images'
files = list_files(directory, "jpg")
for f in files:
    print(f)
    generateColorImage(f)



#bar = plot_colors2(hist, clt.cluster_centers_)

#plt.axis("off")
#plt.imshow(bar)
#plt.show()
